@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2014 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -581,7 +581,7 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv6_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr6, ifa->ifa_flags);
+		    &addr6, ifa->ifa_prefixlen, ifa->ifa_flags);
 		break;
 #endif
 	}
@@ -1062,6 +1062,8 @@ struct nlmr
 };
 
 #ifdef INET
+const char *if_pfname = "Packet Socket";
+
 int
 if_openrawsocket(struct interface *ifp, int protocol)
 {
@@ -1237,7 +1239,7 @@ if_address(const struct interface *iface,
 	nlm.ifa.ifa_prefixlen = inet_ntocidr(*netmask);
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
-	    iface->name, (unsigned short)(strlen(iface->name) + 1));
+	    iface->alias, (unsigned short)(strlen(iface->alias) + 1));
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
 	    &address->s_addr, sizeof(address->s_addr));
 	if (action >= 0 && broadcast)
@@ -1323,8 +1325,10 @@ if_address6(const struct ipv6_addr *ap, int action)
 	struct nlma nlm;
 	struct ifa_cacheinfo cinfo;
 	int retval = 0;
+/* IFA_FLAGS is not a define, but is was added at the same time
+ * IFA_F_NOPREFIXROUTE was do use that. */
 #ifdef IFA_F_NOPREFIXROUTE
-	uint32_t flags;
+	uint32_t flags = 0;
 #endif
 
 	memset(&nlm, 0, sizeof(nlm));
@@ -1337,10 +1341,24 @@ if_address6(const struct ipv6_addr *ap, int action)
 		nlm.hdr.nlmsg_type = RTM_DELADDR;
 	nlm.ifa.ifa_index = ap->iface->index;
 	nlm.ifa.ifa_family = AF_INET6;
+	if (ap->addr_flags & IFA_F_TEMPORARY) {
+#ifdef IFA_F_NOPREFIXROUTE
+		flags |= IFA_F_TEMPORARY;
+#else
+		nlm.ifa.ifa_flags |= IFA_F_TEMPORARY;
+#endif
+	}
+#ifdef IFA_F_MANAGETEMPADDR
+	else if (ap->flags & IPV6_AF_AUTOCONF &&
+	    ip6_use_tempaddr(ap->iface->name))
+		flags |= IFA_F_MANAGETEMPADDR;
+#endif
+
+	/* Add as /128 if no IFA_F_NOPREFIXROUTE ? */
 	nlm.ifa.ifa_prefixlen = ap->prefix_len;
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
-	    ap->iface->name, (unsigned short)(strlen(ap->iface->name) + 1));
+	    ap->iface->alias, (unsigned short)(strlen(ap->iface->alias) + 1));
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
 	    &ap->addr.s6_addr, sizeof(ap->addr.s6_addr));
 
@@ -1353,10 +1371,12 @@ if_address6(const struct ipv6_addr *ap, int action)
 	}
 
 #ifdef IFA_F_NOPREFIXROUTE
-	if (!IN6_IS_ADDR_LINKLOCAL(&ap->addr)) {
-		flags = IFA_F_NOPREFIXROUTE;
+	if (!IN6_IS_ADDR_LINKLOCAL(&ap->addr))
+		flags |= IFA_F_NOPREFIXROUTE;
+#endif
+#ifdef IFA_F_NOPREFIXROUTE
+	if (flags)
 		add_attr_32(&nlm.hdr, sizeof(nlm), IFA_FLAGS, flags);
-	}
 #endif
 
 	if (send_netlink(ap->iface->ctx, NULL,
@@ -1490,6 +1510,15 @@ if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 	return -1;
 }
 
+int
+if_getlifetime6(__unused struct ipv6_addr *ia)
+{
+
+	/* God knows how to work out address lifetimes on Linux */
+	errno = ENOTSUP;
+	return -1;
+}
+
 struct nlml
 {
 	struct nlmsghdr hdr;
@@ -1566,11 +1595,9 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 	snprintf(path, sizeof(path), "%s/%s/autoconf", prefix, ifname);
 	ra = check_proc_int(path);
 	if (ra != 1) {
-		if (!own) {
+		if (!own)
 			syslog(LOG_WARNING,
 			    "%s: IPv6 kernel autoconf disabled", ifname);
-			return -1;
-		}
 	} else if (ra != -1 && own) {
 		if (write_path(path, "0") == -1) {
 			syslog(LOG_ERR, "write_path: %s: %m", path);
@@ -1597,4 +1624,44 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 
 	return ra;
 }
+
+int
+ip6_use_tempaddr(const char *ifname)
+{
+	char path[256];
+	int val;
+
+	if (ifname == NULL)
+		ifname = "all";
+	snprintf(path, sizeof(path), "%s/%s/use_tempaddr", prefix, ifname);
+	val = check_proc_int(path);
+	return val == -1 ? 0 : val;
+}
+
+int
+ip6_temp_preferred_lifetime(const char *ifname)
+{
+	char path[256];
+	int val;
+
+	if (ifname == NULL)
+		ifname = "all";
+	snprintf(path, sizeof(path), "%s/%s/temp_prefered_lft", prefix,
+	    ifname);
+	val = check_proc_int(path);
+	return val < 0 ? TEMP_PREFERRED_LIFETIME : val;
+}
+
+int
+ip6_temp_valid_lifetime(const char *ifname)
+{
+	char path[256];
+	int val;
+
+	if (ifname == NULL)
+		ifname = "all";
+	snprintf(path, sizeof(path), "%s/%s/temp_valid_lft", prefix, ifname);
+	val = check_proc_int(path);
+	return val < 0 ? TEMP_VALID_LIFETIME : val;
+}
 #endif
